@@ -1264,7 +1264,7 @@ typedef struct {
 
 #define OCF_LE_SET_EVENT_MASK			0x0001
 typedef struct {
-	uint64_t	event_mask;
+	uint8_t		mask[8];
 } __attribute__ ((packed)) le_set_event_mask_cp;
 #define LE_SET_EVENT_MASK_CP_SIZE 8
 
@@ -1457,7 +1457,7 @@ typedef struct {
 	uint16_t	handle;
 	uint8_t		key[16];
 } __attribute__ ((packed)) le_ltk_reply_cp;
-#define LE_LTK_REQUEST_REPLY_CP_SIZE 18
+#define LE_LTK_REPLY_CP_SIZE 18
 typedef struct {
 	uint8_t		status;
 	uint16_t	handle;
